//+------------------------------------------------------------------+
//|                                                        Gauss.mq5 |
//|                              Copyright  2015, Vladimir Karputov |
//|                                           http://wmua.ru/slesar/ |
//+------------------------------------------------------------------+
#property copyright "Copyright  2015, Vladimir Karputov"
#property link      "http://wmua.ru/slesar/"
#property version   "1.11"
#property description "Blur image using a Gaussian filter"
#property script_show_inputs
#include <Canvas\Canvas.mqh>
#include <Math\Alglib\alglib.mqh>
//--- input
input uint  radius=1;   // radius blur
//---
int         NNodes;     // 
//+------------------------------------------------------------------+
//| version 1.10:                                                    |
//| New: horizontal and vertical blur                                |
//| version 1.11:                                                    |
//| New: application of the Gauss formulas                           |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Script program start function                                    |
//+------------------------------------------------------------------+
void OnStart()
  {
   NNodes=(int)radius*2+1;
//--- creating a resource
   ResetLastError();
   if(!ResourceCreate("res","\\Files\\Images\\screen.bmp"))
     {
      Print("Error creating a resource ",GetLastError());
      return;
     }
   uint res_data[];                    // array for receiving data from the graphical resource
   uint res_width=0;                   // width of the resource
   uint res_height=0;                  // height of the resource
//+------------------------------------------------------------------+
//| reads data from the graphical resource                           |
//+------------------------------------------------------------------+
   ResetLastError();
   if(!ResourceReadImage("::res",res_data,res_width,res_height))
     {
      Print("Error reads data from the graphical resource ",GetLastError());
      Print("attempt number two");
      //--- attempt number two: now the picture width and height are known
      ResetLastError();
      if(!ResourceReadImage("::res",res_data,res_width,res_height))
        {
         Print("Error reads data from the graphical resource ",GetLastError());
         return;
        }
     }
//---
   if(radius>=res_width/2 || radius>=res_height/2)
     {
      Print("Error! Image size is very small or very large blur");
      return;
     }
//+------------------------------------------------------------------+
//| decomposition of pictures on the components r, g, b              |
//+------------------------------------------------------------------+
   int      size=ArraySize(res_data);
   uint     res_blur_hor[];               // temp array for receiving data from the graphical resource
   uint     res_blur_ver[];               // temp array for receiving data from the graphical resource
   if(ArrayResize(res_blur_hor,size)==-1)
     {
      Print("Error setting the new size of an array of \"res_blur_hor\"");
      return;
     }
   ArrayInitialize(res_blur_hor,0);

   if(ArrayResize(res_blur_ver,size)==-1)
     {
      Print("Error setting the new size of an array of \"res_blur_ver\"");
      return;
     }
   ArrayInitialize(res_blur_ver,0);

   uint     clr,clr_temp;                 // color in ARGB format
   uchar    r_data[],g_data[],b_data[];   // arrays of colors R, G, B
   if(ArrayResize(r_data,size)==-1)
     {
      Print("Error setting the new size of an array of \"r_data\"");
      return;
     }

   if(ArrayResize(g_data,size)==-1)
     {
      Print("Error setting the new size of an array of \"g_data\"");
      return;
     }

   if(ArrayResize(b_data,size)==-1)
     {
      Print("Error setting the new size of an array of \"b_data\"");
      return;
     }

//---
   double weights[];       // array of weights
   GetQuadratureWeights(1,NNodes,weights);

//+------------------------------------------------------------------+
//| blur horizontal (axis X)                                         |
//+------------------------------------------------------------------+
   for(int i=0;i<size;i++)
     {
      clr_temp=res_data[i];
      clr=COLOR2RGB(clr_temp);
      r_data[i]=GETRGBR(clr_temp);
      g_data[i]=GETRGBG(clr_temp);
      b_data[i]=GETRGBB(clr_temp);
     }

   uint XY;                                // pixel coordinate in the array
   for(uint Y=0;Y<res_height;Y++)
     {
      for(uint X=radius;X<res_width-radius;X++)
        {
         XY=Y*res_width+X;
         double   r_temp=0.0,g_temp=0.0,b_temp=0.0;
         int      coef=0;
         int      j=NNodes/2;
         for(int i=-1*j;i<j+1;i=i+1)
           {
            r_temp=r_temp+r_data[XY+i]*weights[coef];
            g_temp=g_temp+g_data[XY+i]*weights[coef];
            b_temp=b_temp+b_data[XY+i]*weights[coef];
            coef=coef+1;
           }
         r_data[XY]=(uchar)r_temp;
         g_data[XY]=(uchar)g_temp;
         b_data[XY]=(uchar)b_temp;
         clr_temp=XRGB(r_data[XY],g_data[XY],b_data[XY]);
         res_blur_hor[XY]=clr_temp;
        }
     }
//+------------------------------------------------------------------+
//| blur vertically (axis Y)                                         |
//+------------------------------------------------------------------+
   for(int i=0;i<size;i++)
     {
      clr_temp=res_blur_hor[i];
      clr=COLOR2RGB(clr_temp);
      r_data[i]=GETRGBR(clr_temp);
      g_data[i]=GETRGBG(clr_temp);
      b_data[i]=GETRGBB(clr_temp);
     }

   for(uint X=0;X<res_width;X++)
     {
      for(uint Y=radius;Y<res_height-radius;Y++)
        {
         XY=Y*res_width+X;
         double   r_temp=0.0,g_temp=0.0,b_temp=0.0;
         int      coef=0;
         int      j=NNodes/2;
         for(int i=-1*j;i<j+1;i=i+1)
           {
            int dxdy=i*(int)res_width;
            r_temp=r_temp+r_data[XY+dxdy]*weights[coef];
            g_temp=g_temp+g_data[XY+dxdy]*weights[coef];
            b_temp=b_temp+b_data[XY+dxdy]*weights[coef];
            coef=coef+1;
           }
         r_data[XY]=(uchar)r_temp;
         g_data[XY]=(uchar)g_temp;
         b_data[XY]=(uchar)b_temp;
         clr_temp=XRGB(r_data[XY],g_data[XY],b_data[XY]);
         res_blur_ver[XY]=clr_temp;
        }
     }
   ResetLastError();
   if(!ResourceCreate("res_a",res_blur_ver,res_width,res_height,0,0,0,COLOR_FORMAT_ARGB_NORMALIZE))
     {
      Print("Error creating a resource \"res_a\"",GetLastError());
      return;
     }
   ResetLastError();
   if(!ResourceSave("::res_a","\\Images\\screen1.bmp"))
     {
      Print("Error save resourse \"res_a\"",GetLastError());
      return;
     }
//---
   ResourceFree("res");
   ResourceFree("res_a");
  }
//+------------------------------------------------------------------+
//| Gets array of quadrature weights                                 |
//+------------------------------------------------------------------+
void GetQuadratureWeights(const double mu0,const int n,double &w[])
  {
   CAlglib alglib;
   double      alp[];
   double      bet[];
   ArrayResize(alp,n);
   ArrayResize(bet,n);
   ArrayInitialize(alp,1);
   ArrayInitialize(bet,1);

   double      out_x[];
//double      out_w[];
   int         inf;

   alglib.GQGenerateRec(alp,bet,mu0,n,inf,out_x,w);
  }
//+------------------------------------------------------------------+
